#!/usr/bin/env python
import roslib, rospy
import numpy as np
import scipy.spatial.distance as dist
from laser_clustering.msg import ClustersMsg
from radio_services.srv import InstructionWithAnswer

trace_array = []
trace_count = False
trace_results = []
cls_results = []
#traced_clusters attribute: is used to store the points of the traced clusters in a slide window manner
# -> format: [ CL1=[[pTW1], [pTW2], ...] ,CL2=[[pTW1],[pTW2],...] , ... ]
#    where 
#	- CLn: indicates a cluster in the slide window
#	- pTWn: indicates the points of the specific cluster in the timewindow n (TWn)
#Therefore CL1, which represents cluster 1, is a list of points for <max_num_slide_window> timewindow
traced_clusters = []

#num_clusters: it is an array that defines the number of clusters that consist a traced cluster.
num_clusters = []

max_cls = 0
max_num_slide_window = 4
first_trace = True
clusters_publisher = None
frame_id = ''
dt = 25;#period in ms (dt between scans)
speed_ = 5;#human walking speed in km/h
z_scale = float(speed_*dt) / float(3600)

running = False
clustering_sub = None
input_clusters_topic = ''

def init():
    global clusters_publisher, frame_id, dt, speed_, z_scale
    global running, clustering_sub, input_clusters_topic

    rospy.init_node('laser_overlap_trace')

    input_clusters_topic = rospy.get_param('~input_clusters_topic','laser_clustering/clusters')
    output_clusters_topic = rospy.get_param('~output_clusters_topic','~clusters')
    frame_id = rospy.get_param('~frame_id','laser_link')
    dt = rospy.get_param('~dt', 25)
    speed_ = rospy.get_param('~human_speed', 5)
    running = rospy.get_param('~run_on_startup', False)

    z_scale = float(speed_*dt) / float(3600)

    rospy.Service('/human_pattern_recognition/laser_overlap_trace/node_state_service', InstructionWithAnswer, nodeStateCallback)

    if running:
        clustering_sub = rospy.Subscriber(input_clusters_topic, ClustersMsg, overlap_trace)

    clusters_publisher = rospy.Publisher(output_clusters_topic, ClustersMsg, queue_size=10)
    while not rospy.is_shutdown():
        rospy.spin()


#Get the mean point (x,y) of a set of laser-scan points.
# -> choice parameter declares the first (True) or the last (False) part of cluster
def get_centroid(cluster, choice):

    if choice == False:
        z_filter = np.where(cluster[2]==cluster[2][len(cluster[2])-1])
    else:
        z_filter = np.where(cluster[2]==cluster[2][0])

    mean_x = np.mean(cluster[0][z_filter])
    mean_y = np.mean(cluster[1][z_filter])
    mean_z = np.mean(cluster[2][z_filter])

    return np.array((mean_x, mean_y, mean_z))


def create_trace():

    global trace_results, cls_results
    global traced_clusters, max_cls
    global num_clusters

    temp = []
    counter = 1
    num_clusters = np.zeros(max_cls, dtype=int)

    #for each cluster
    for j in range(0,max_cls):
        for i in range(0, len(trace_results)):

            if len(trace_results[i]) > counter:
                counter = counter +1
       
                try:
                    temp.append(cls_results[i][trace_results[i].index(-1)])
                except ValueError:
                   if j in trace_results[i]:
                        print 'ValueError exception: with trace_results = {} and j = {}'.format(trace_results, j)
                        temp.append(cls_results[i][trace_results[i].index(j)])
                   else:
                        continue

                num_clusters[j] = num_clusters[j] + 1
            if j in trace_results[i]:
                temp.append(cls_results[i][trace_results[i].index(j)])
                num_clusters[j] = num_clusters[j] + 1

        traced_clusters.append(temp)
        temp=[]

def continue_trace():

    global trace_results, cls_results
    global traced_clusters
    global num_clusters

    last_element = len(trace_results) - 1

    for i in range(0, len(trace_results[last_element])):
        index = trace_results[last_element][i]

        if index != -1:
            if len(traced_clusters[index]) == max_num_slide_window:
                del traced_clusters[index][0]
                num_clusters[index] = num_clusters[index] - 1

            traced_clusters[index].append(cls_results[last_element][i])
            num_clusters[index] = num_clusters[index] + 1
        else:
            traced_clusters.append([cls_results[last_element][i]])
            num_clusters = np.append(num_clusters, 1)



#Tracks the human walks by fitting the new set of points(frame) to the closest cluster.
#First it finds the centroids of the last part(frame) of the previous clusters.
#Then it connects a new cluster to a previous one, by getting the minimum euclidean distance of the new cluster's centroid and the previous clusters' centroids.
#Finally, the function calculates an array of trace results, where:
#	a) each index denotes the number of a new coming cluster
#	b) the value at each index denotes the number of a previous cluster that most likely fits
def overlap_trace(clusters_msg):
    global trace_array  #the centroid point of each cluster at every scan
    global trace_count, max_num_slide_window

    global trace_results  #the position of the clusters at each scan
    global cls_results   #the set of points of each cluster at every scan
    global traced_clusters, first_trace, max_cls
    global frame_id, clusters_publisher
    global num_clusters

    cls = []

    error = 100
    min_dist = -1.0
    index = 0
    list_dist = []
    temp_list = []
    new_cluster = True

    xi = np.array(clusters_msg.x)
    yi = np.array(clusters_msg.y)
    zi = np.array(clusters_msg.z)

    scan_time = clusters_msg.scan_time

    array_sizes = np.array(clusters_msg.array_sizes)

    prev_index = 0

    if(len(array_sizes) > 0):


        for i in range(0, len(array_sizes)):
            xk = []
            yk = []
            zk = []
            for j in range(prev_index, prev_index+array_sizes[i]-1):
                xk.append(xi[j])
                yk.append(yi[j])
                zk.append(zi[j])
            cls.append([np.array(xk), np.array(yk), np.array(zk)])
            prev_index = array_sizes[i]-1

        if len(trace_array) == 0:
            for i in range(0, len(cls)):
                trace_array.append(get_centroid(cls[i], False))
                trace_results.append([i])
                cls_results.append([cls[i]])
          
        else:
            #condition where the clusters have been reduced
            if trace_count:
                num_cl = np.where(num_clusters == 0)
                num_clusters = np.delete(num_clusters, num_cl, 0)

                traced_clusters2 = [x for x in traced_clusters if x != []]
                traced_clusters = list(traced_clusters2)
                trace_count = False
	
            if len(cls) > len(trace_array):
                first = trace_array
                second = cls
                new_cluster = True
            else:
                first = cls
                second = trace_array
                new_cluster = False

            for i in range(0, len(first)):
                if new_cluster == False:
                    coord = get_centroid(first[i], True)

                for j in range(0, len(second)):
                #eucl_dist for every combination
                    if new_cluster:
                        coord = get_centroid(second[j], True)
                        d = dist.euclidean(coord,first[i])
                        temp_list.append(d)
                    else:
                        d = dist.euclidean(coord,second[j])
                        temp_list.append(d)

                list_dist.append(temp_list)

                temp_list = []

            min_val = -1.0
            row = 0
            num = 0
            temp_num = 0
            col = -1
            results = []

            temp_list = list(list_dist)
            length = len(list_dist)

            for i in range(0,len(cls)):
                results.append(-1)

            while num<length:
                for i in range(0, len(temp_list)):
                    if min_val == -1.0:
                        min_val = min(temp_list[i])
                        temp_row = i
                    else:
                        if min_val > min(temp_list[i]):
                            min_val = min(temp_list[i])
                            temp_row = i

                for i in range(0, len(list_dist)):
                    if min_val in list_dist[i]:
                        row = i
                        break

                if new_cluster:
                    results[list_dist[row].index(min_val)] = row

                else:
                    results[row] = list_dist[row].index(min_val)

                ind = temp_list[temp_row].index(min_val)
                del temp_list[temp_row]

                for i in range(0, len(temp_list)):
                    temp_list[i][ind] = error

                num = num + 1
                col = -1
                row =0
                min_val = -1.0

            # a cluster disappears
            if (len(results) < len(trace_array) and len(traced_clusters) != 0):
                rm_list = []
                #remove the unnecessary clusters
                for j in range(0, len(trace_array)):
                    if j not in results:
                        rm_list.append(j)

                for i in rm_list:
                    del traced_clusters[i][:]
                    num_clusters[i] = 0
                    
                    trace_count = True

            #remove previous and add the new ones
            del trace_array[:]
            for i in range(0, len(cls)):
                trace_array.append(get_centroid(cls[i], False))

            trace_results.append(results)
            cls_results.append(cls)

            #the maximum number of clusters
            if max_cls < len(results):
                max_cls = len(results)

            if len(trace_results) == max_num_slide_window:
                if first_trace:
                    create_trace()
                    first_trace = False
                else:
                    continue_trace()

                final_clusters, index_clusters = getClusterSet()
                x_ = []
                y_ = []
                z_ = []
                arr_sz = []
                for i in range(0, len(final_clusters)):
                    sz = len(final_clusters[i][0])
                    arr_sz.append(sz)
                    for j in range(0, sz):
                        x_.append(final_clusters[i][0][j])
                        y_.append(final_clusters[i][1][j])
                        z_.append(final_clusters[i][2][j])

                cls_msg = ClustersMsg()
                cls_msg.header.stamp = rospy.Time.now()
                cls_msg.header.frame_id = frame_id
                cls_msg.x = x_
                cls_msg.y = y_
                cls_msg.z = z_
                cls_msg.array_sizes = arr_sz
                cls_msg.scan_time = scan_time
                cls_msg.num_clusters = index_clusters
                clusters_publisher.publish(cls_msg)

                del trace_results[0]
                del cls_results[0]
                max_cls = len(results)
    else:
        del trace_results[:]
        del cls_results[:]
        del traced_clusters[:]
        del trace_array[:]
        num_clusters = []
        first_trace = True
        trace_count = False
        max_cls =0
        clusters_publisher.publish(clusters_msg)


#combines the points of each tracked cluster.
#returns a list of the trackedCluster points
def getClusterSet():
    global traced_clusters
    global z_scale

    index_clusters = []
    final_clusters = []

    #for every tracked cluster
    for i in range(0, len(traced_clusters)):
        if len(traced_clusters[i]) == 0:
             index_clusters.append(0)
             continue

        #initialize x,y,z of trackedCluster_i
        xar = np.array(traced_clusters[i][0][0])
        yar = np.array(traced_clusters[i][0][1])
        zar = np.array(traced_clusters[i][0][2])

	index_clusters.append(len(xar))

        #z dimension changes: 
        #Each timewindow resets the time, thus the time (z dimention) begins from 0.0.
        #In order to combine the points of the available timewindows, the time should increment periodically in each set of timewindow points.
        for j in range(1, len(traced_clusters[i])):
             xar = np.append(xar,traced_clusters[i][j][0])
             yar = np.append(yar,traced_clusters[i][j][1])
             B = traced_clusters[i][j][2].copy()
           
             index_clusters.append(len(traced_clusters[i][j][0]))

             increment = zar[len(zar) - 1] + z_scale
             B[::1]  += increment
             zar =  np.append(zar,B)	

        final_clusters.append([xar,yar,zar])

    return final_clusters, index_clusters

##
## @brief      This function is called when the corresponding service is called
##             and based on the parameter passed, either changes its state and
##             returns the new state, or just returns the current state.
##             0: Change the current node state to WAITING (false) and return the current node state.
##             1: Change the current node state to RUNNING (true), return the current node state (and do not change the mode).
##            -1: Return the current node state.
##
## @param      req   The requested action
##
## @return     The current state of the node
##
def nodeStateCallback(req):
    global running, clustering_sub, input_clusters_topic
    global trace_array, trace_count, trace_results, cls_results
    global traced_clusters, num_clusters, max_cls, first_trace
    if req.command == 0 and running:
        running = False
        clustering_sub.unregister()
        print 'Stopped laser overlap trace!'
    elif req.command == 1 and not running:
        running = True
        trace_array = []
        trace_count = False
        trace_results = []
        cls_results = []
        traced_clusters = []
        num_clusters = []
        max_cls = 0
        first_trace = True
        clustering_sub = rospy.Subscriber(input_clusters_topic, ClustersMsg, overlap_trace)
        print 'Started laser overlap trace!'
    return running

if __name__ == '__main__':
    init() 
