#' Builds a query that specifies the parameters of the statistic.
#'
#' Transforms the <Parameters> that describe the data/samples of the statistic into a suitable "query format" with choices and filters.
#'
#' @param params The <Parameters> that describe the statistic.
#' @param map The arguments of the statistic
#' @param cond Specifies the type of the transformed query. 'n': default for normal statistics, 'r': for linear/correlation statistics
#' @return The transformed query-parameters
.buildQuery <- function(params, map=NULL, cond='n') {

  if(cond == 'n') {
    return(.setInputSimple(params, map))
  }
  else if(cond == 'r') {
    return(.setInputRelation(params, map))
  }


}

#set the input of the statistics - convertion in QfrMessage(s).
.setInputSimple <- function(params, map) {

  # the input parameter should contain all the parameteres set for the specific statistic
  input <- .jnew("gr.demokritos.iit.radio.home.entities.StatParams")

  if(!is.null(map)) {
    .jrcall(input, "setInput", map)
  }

  #for select...
  dependency <- getDependency(params)

  selectList <- .jnew("java.util.ArrayList")
  for(i in 1:length(dependency)) {
    .jrcall(selectList, "add", as.character(dependency[i]))
  }

  if(length(getGroups(params)) == 0) {
    msg <- .jnew("gr.demokritos.iit.radio.home.qfr.QfrMessage")
    .jrcall(msg, "setSelect", selectList)
    .jrcall(input, "addQfr", msg)
  }

  else {
    for (group in getGroups(params)) {
      #set the QfrMessage(s) . Each one has the same select part.
      msg <- .jnew("gr.demokritos.iit.radio.home.qfr.QfrMessage")
      .jrcall(msg, "setSelect", selectList)
      msg <- .buildWhere(group, msg)
      .jrcall(input, "addQfr", msg)
    }
  }

  return(input)

}

.buildWhere <- function(group, msg) {

  triples <- getTriples(group)

  for (triple in triples) {
    vect <- .jnew("java.util.Vector")

    for(val in triple) {
      .jrcall(vect, "add", val)
    }
    .jcall(msg, "V","addTriple", vect)

  }

  return(msg)
}

#Set the input for relational statistic functions , like linear regression, correlation etc
.setInputRelation <- function(params, map) {

  # the input parameter contains the above specified objects
  input <- .jnew("gr.demokritos.iit.radio.home.entities.StatParams")

  if(!is.null(map)) {
    print(map)
    .jrcall(input, "setInput", map)
  }

  #for select
  cor <- getDependency(params)

  selectList <- .jnew("java.util.ArrayList")

  for(i in 1:length(cor)) {
    selectList <- .jnew("java.util.ArrayList")

    msg <- .jnew("gr.demokritos.iit.radio.home.qfr.QfrMessage")
    .jrcall(selectList, "add", as.character(cor[i]))
    .jrcall(msg, "setSelect", selectList)
    .jrcall(input, "addQfr", msg)
  }

  return(input)

}
