#' Correlation Coefficient statistic with privacy preserving
#'
#' Correlation Coefficient statistic that preserves the privacy of the data sources. The statistic is performed for exactly two variables.
#'
#' @param params The <Parameters> that should be defined to compute the statistic. The list of dependency field should have exactly the two variables for the correlation test.
#' @return A double that represents the correlation result (range: [-1.0, 1.0] ).
#' @examples
#' We want to apply a correlation test between the 2 variables: value_time_before, value_time_after.
#' Firstly we should define the variables through the <Parameters>.
#' The first element in the Dependent list defines how much the variable is correlated to the second one:
#' corParam <- Parameters( list("value_time_before", "value_time_after") , NULL)
#' cor(corParam)
#'
#' @export
cor <- function(params)
{

  if(missing(params)) {
    stop("The argument 'params' should be filled in.")
  }

  if(class(params)[2] != "Parameters") {
    stop("The argument 'params' should be an instance of 'Parameters'.")
  }

  if(length(getDependency(params)) != 2) {
    stop("The correlation coeeficient statistic can be computed over exactly two variables.")
  }


  input <- .buildQuery(params,cond = 'r')

  testObj <- .jnew("gr.demokritos.iit.radio.home.orchestration.Controller")
  result <- .jcall(testObj, "D", "corCoefParam", input)

  #remove later!!!
  #the results should be displayed in the end
  p1 <- proc.time()
  Sys.sleep(0.2)
  proc.time() - p1
  .convertCorCoefResult(result)

  #print(result)

}

.convertCorCoefResult <- function(result) {
  r = round(result, digits = 4)
  aList <- list(r)

  return(aList)
}
