#' Group for statistics
#'
#' Creates a group that indicates the characteristics of a sample that is needed for a statistic computation.
#' @param tripleList list of triples (<variable> <operator> <value>) that specify the group.
#' @return the initialized group
#' @export

GroupStat <- function(tripleList = NULL)
{

  if(typeof(tripleList) != "list" && !is.null(tripleList)) {
    stop("The argument should be a list of triples.")
  }

  for(i in tripleList) {
    if(length(tripleList[i]) != 3) {
      stop("The argument should be a list of triples.")
    }
  }

  str <- list(
    hasTriples = tripleList
  )

  class(str) <- append(class(str), "GroupStat")


  return(str)
}

################### GETTER - SETTER ################################

#' Setter for triples in a group
#'
#' Sets the triples to a <GroupStat>
#' @param obj The <GroupStat> object
#' @param value The list of triples that the user wants to set
#' @return The updated object
#' @export
#'
setTriples <- function(obj, value)
{
  UseMethod("setTriples",obj)

}

setTriples.default <- function(obj, value)
{
  print("I do not know how to handle this object.")
  return(obj)
}

setTriples.GroupStat <- function(obj, value)
{
  for(i in value) {
    if(length(value[i]) != 3) {
      stop("The argument should be a list of triples.")
    }
  }

  obj$hasTriples <- value
  return(obj)
}

#' Getter for triples in a group
#'
#' @param obj The <GroupStruct> object where the triples belong
#' @return The list of triples in the specific object
#' @export
#'
getTriples <- function(obj)
{
  UseMethod("getTriples",obj)
}

getTriples.default <- function(obj)
{
  print("I do not know how to handle this object.")
  return(NULL)
}


getTriples.GroupStat <- function(obj)
{
  return(obj$hasTriples)
}
