
#' Mean statistic with privacy preserving
#'
#' Mean (Average) statistic that preserves the privacy of the data sources. The statistic is performed for exactly one sample.
#'
#' @param params The <Parameters> that should be defined to compute the statistic.
#' @return A double that represents the average result
#' @examples
#' You want the mean of a single variable like the 'value_time_before'
#' Firstly www should define it through the dependent var of the <Parameters>:
#' mParam <- Parameters( list("value_time_before") , NULL)
#' mean(mParam)
#'
#' Also we may want the mean of the variable 'value_time_before' only for the male population (eg. sex=M).
#' The variable should be specified into the DP list , while all the conditions should be declared through a group.
#' mGr <- GroupStat( list( c("sex","=","M")) )
#' mParam <- Parameters( list("value_time_before") , list(mGr) )
#' mean(mParam)
#' @export
mean <- function(params)
{

  if(missing(params)) {
    stop("The argument 'params' should be filled in.")
  }

  if(class(params)[2] != "Parameters") {
    stop("The argument 'params' should be an instance of 'Parameters'.")
  }

  if(length(getDependency(params)) != 1) {
    stop("The mean can be computed over a single dependency variable.")
  }


  input <- .buildQuery(params)

  testObj <- .jnew("gr.demokritos.iit.radio.home.orchestration.Controller")
  result <- .jcall(testObj, "D", "avgParam", input)

  #remove later!!!
  #the results should be displayed in the end
  p1 <- proc.time()
  Sys.sleep(0.2)
  proc.time() - p1
  .convertMeanResult(result)

}


.convertMeanResult <- function(result) {
  r = round(result, digits = 2)
  aList <- list(r)

  return(aList)
}
