#' Parameters for statistics
#'
#' Defines the parameters for the statistic computation, such as the list of groups <GroupStat>
#' and the attributes that need to be correlated by the statistic.
#' @param dependent The list of dependent variables
#' @param groups The list of <GroupStat>
#' @return The initialized parameters
#' @export
Parameters <- function(dependent=NULL, groups=NULL)
{

  if(typeof(groups) != "list" && !is.null(groups)) {
    stop("The argument 'groups' should be a list of of 'GroupStat' instances.")
  }

  for(gr in groups) {

    if(length(class(gr)) == 1 || class(gr)[2] != "GroupStat") {
      stop("The argument 'groups' should be a list of 'GroupStat' instances.")
    }
  }


  #if(typeof(correlation) != "list")
  #{
   # stop("The 'correlation' parameter should be a vector of Strings.")
  #}

  str <- list(
    hasDependency = dependent,
    hasGroups = groups
  )

  ## Set the name for the class
  class(str) <- append(class(str),"Parameters")

  return(str)
}

#################### SETTERS ######################

#' Setter for dependent attribute
#'
#' @param obj The <Parameters> object
#' @param dependent The list of dependent variables
#' @param the updated object
#' @export
setDependency <- function(obj, dependent)
{
  #print("Calling the base setDependency function")

  UseMethod("setDependency",obj)

}

setDependency.default <- function(obj, dependent)
{
  print("I do not know how to handle this object.")
  return(obj)
}

setDependency.Parameters <- function(obj, dependent)
{
  #print("In setDependency.Parameters and setting the value")
  obj$hasDependency <- dependent
}

#' Setter for groups attribute
#'
#' @param obj The <Parameters> object
#' @param groups The list of contributed groups
#' @param the updated object
#' @export
setGroups <- function(obj, groups)
{
  #print("Calling the base setGroups function")

  if(typeof(groups) != "list" && !is.null(groups)) {
    stop("The argument 'groups' should be a list of of 'GroupStat' instances.")
  }

  for(gr in groups) {
    if(length(class(gr)) == 1 || class(gr)[2] != "GroupStat") {
      stop("The argument 'groups' should be a list of 'GroupStat' instances.")
    }
  }

  UseMethod("setGroups", obj)

}

setGroups.default <- function(obj, groups)
{
  print("I do not know how to handle this object.")
  return(obj)
}

setGroups.Parameters <- function(obj, groups)
{
  #print("In setGroups.Parameters and setting the value")
  obj$hasGroups <- groups
}

####################### GETTERS ##############################

#' Getter for dependent variables in <Parameters>
#'
#' @param obj The <Parameters> object where the list of dependent variables belong
#' @return The list of correlations in the specific object
#' @export
#'
getDependency <- function(obj)
{
 #print("Calling the base getDependency function")
  UseMethod("getDependency",obj)
}

getDependency.default <- function(obj)
{
  print("I do not know how to handle this object.")
  return(NULL)
}


getDependency.Parameters <- function(obj)
{
  #print("In getDependency and returning the value")
  return(obj$hasDependency)
}

#' Getter for groups
#'
#' @param obj The <Parameters> object where the groups belong
#' @return The list of groups in the specific object
#' @export
getGroups <- function(obj)
{
  #print("Calling the base getGroups function")
  UseMethod("getGroups",obj)

}

getGroups.default <- function(obj)
{
  print("I do not know how to handle this object.")
  return(NULL)
}


getGroups.Parameters <- function(obj)
{
  #print("In getGroups and returning the value")
  return(obj$hasGroups)
}

