#' Plot the Normal Distribution Bell with privacy preserving
#'
#' Plots the gaussian bell with the use of mean and st.deviation while it preserves the privacy of the data sources. The method is performed for exactly a single sample.
#'
#' @param params The <Parameters> that should be defined to compute the statistic.
#' @examples
#' We want to plot the Gaussian distribution of the variable 'value_time_after' for the sample group where the people are female (sex=F).
#' Firstly we should define the <Parameters> with the DV and the IV:
#' nGr <- GroupStat( list( c("sex", "=", "F") ))
#' nParam <- Parameters( list("value_time_after") , list(nGr))
#' plotnorm(nParam)
#' @export
plotnorm <- function(params)
{

  if(missing(params)) {
    stop("The argument 'params' should be filled in.")
  }

  if(class(params)[2] != "Parameters") {
    stop("The argument 'params' should be an instance of 'Parameters'.")
  }

  if(length(getGroups(params)) != 1) {
    stop("The normal distribution can be computed over a single sample group.")
  }

  sigma = stdev(params)[[1]]
  mi = mean(params)[[1]]

  l = length(sigma)
  cat('standard deviation: ',sigma)
  cat(' - mean: ',mi,'\n')


  varX = seq(mi-(sigma*3),mi+(sigma*3), length=1000)
  varY = dnorm(varX, mean=mi, sd=sigma)

  plot(varX,varY,type="l",lwd=1)

}
