#' Standard Deviation statistic with privacy preserving
#'
#' Standard Deviation statistic that preserves the privacy of the data sources. The statistic is performed for exactly a single sample.
#'
#' @param params The <Parameters> that should be defined to compute the statistic.
#' @return A double that represents the standard deviation result
#' @examples
#' We want the st. deviation of a single variable like the 'value_time_before'
#' Firstly we should define it through the dependent list of the <Parameters>:
#' stParam <- Parameters( list("value_time_before") , NULL )
#' stdev(stParam)
#'
#' Also we may want the st.deviation of the variable 'value_time_before' only for the male population (eg. sex=M).
#' The variable should be specified into the DP list , while all the conditions should be declared through a group.
#' stGr <- GroupStat( list( c("sex","=","M")) )
#' stParam <- Parameters( list("value_time_before") , list(stGr) )
#' stdev(stParam)
#' @export
stdev <- function(params)
{

  if(missing(params)) {
    stop("The argument 'params' should be filled in.")
  }

  if(class(params)[2] != "Parameters") {
    stop("The argument 'params' should be an instance of 'Parameters'.")
  }

  if(length(getDependency(params)) != 1) {
    stop("The standard deviation statistic can be computed over a single dependency variable.")
  }


  input <- .buildQuery(params)
  print(input)

  testObj <- .jnew("gr.demokritos.iit.radio.home.orchestration.Controller")
  result <- .jcall(testObj, "D", "stdevParam", input)

  #remove later!!!
  #the results should be displayed in the end
  p1 <- proc.time()
  Sys.sleep(0.2)
  proc.time() - p1
  .convertStdevResult(result)

}

.convertStdevResult <- function(result) {
  r = round(result, digits = 5)
  aList <- list(r)

  return(aList)
}
