#' Variance statistic with privacy preserving
#'
#' Variance statistic that preserves the privacy of the data sources. The statistic is performed for exactly a single sample.
#'
#' @param params The <Parameters> that should be defined to compute the statistic.
#' @return A double that represents the variance result
#' @examples
#' We want the variance of a single variable like the 'value_time_before'
#' Firstly we should define it through the dependent list of the <Parameters>:
#' vParam <- Parameters( list("value_time_before") , NULL)
#' var(vParam)
#'
#' Also we may want the variance of the variable 'value_time_before' only for the male population (eg. sex=M).
#' The variable should be specified into the DP list , while all the conditions should be declared through a group.
#' vGr <- GroupStat( list( c("sex","=","M")) )
#' vParam <- Parameters( list("value_time_before") , list(vGr) )
#' var(stParam)
#' @export
var <- function(params)
{
  if(missing(params)) {
    stop("The argument 'params' should be filled in.")
  }

  if(class(params)[2] != "Parameters") {
    stop("The argument 'params' should be an instance of 'Parameters'.")
  }

  if(length(getDependency(params)) != 1) {
    stop("The variance statistic can be computed over a single dependency variable.")
  }



  input <- .buildQuery(params)
  print(input)

  testObj <- .jnew("gr.demokritos.iit.radio.home.orchestration.Controller")
  result <- .jcall(testObj, "D", "varParam", input)

  #remove later!!!
  #the results should be displayed in the end
  p1 <- proc.time()
  Sys.sleep(0.2)
  proc.time() - p1
  .convertVarResult(result)

  #print(result)

}

.convertVarResult <- function(result) {
  r = round(result, digits = 5)
  aList <- list(r)

  return(aList)
}
