% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anova.R
\name{anova}
\alias{anova}
\title{ANOVA statistic with privacy preserving}
\usage{
anova(params, method = "b")
}
\arguments{
\item{params}{The <Parameters> that should be defined to compute the statistic.}

\item{method}{The method of anova that will be applied. Ex. 'b' is for one-way between subjects, 'r' is for repeated measures and 'm' for mixed.}
}
\value{
A set of resulted components (summary table) after the ANOVA execution
}
\description{
ANOVA statistic that preserves the privacy of the data sources. The statistic is performed for at least two samples.
}
\examples{
Example for one-way anova between participants:
We want to apply an one-way anova between participants
on the dependend variable 'value_time_before' and on two groups regarding their gender (sex=M/F).
Firstly we should define the <Parameters>:
gr1 <- GroupStat( list( c("sex", "=", "M") ))
gr2 <- GroupStat( list( c("sex", "=", "F") ))
btwParam <- Parameters( list("value_time_before") , list(gr1, gr2))

To run the one-way BP:
anova(btwParam) or anova(btwParam, "b")

Example for one-way anova within participants or one-way repeated anova:
We want to apply an one-way anova within participants
on the dependend variable 'value' and on two conditions regarding the time (time=before/after).
Firstly we should define the <Parameters>:
gr1 <- GroupStat( list( c("time", "=", "before") ))
gr2 <- GroupStat( list( c("time", "=", "after") ))
repParam <- Parameters( list("value") , list(gr1, gr2))

To run the one-way WP:
anova(repParam, "r")

Example for the mixed anova:
We want to apply a mixed anova
on the dependend variable 'value', on two groups regarding their gender (sex=M/F)
and on two conditions regarding the time (time=before/after).
Firstly we should define the <Parameters>. Group 1 should contain the groups, while Group 2 the conditions:
gr1 <- GroupStat( list( c("sex", "=", "M"), c("sex", "=", "F") ))
gr2 <- GroupStat( list( c("time", "=", "before"), c("time", "=", "after") ))
mixParam <- Parameters( list("value") , list(gr1, gr2))

To run the Mixed Anova:
anova(mixParam, "m")
}
